import datetime
import logging
import threading

from typing import Dict, List, Optional

from .InternalLimiter import InternalLimiter
from .Limits import LimitCollection, RawLimit

LOG = logging.getLogger(__name__)


class HeaderBasedLimiter(InternalLimiter):
    def __init__(self, limit_header: str, count_header: str, friendly_name: str = None):
        super().__init__()
        self._limit_header = limit_header
        self._count_header = count_header
        self._friendly_name = friendly_name

        self._limits: Dict[str, LimitCollection] = {}
        self._limits_lock = threading.Lock()

    @property
    def friendly_name(self) -> str:
        return self._friendly_name

    def _get_limit_scope(
        self, region: str, endpoint_name: str, method_name: str
    ) -> str:
        return ""

    def __get_limit(
        self, region: str, endpoint_name: str, method_name: str
    ) -> LimitCollection:
        scope = self._get_limit_scope(region, endpoint_name, method_name)

        if scope not in self._limits:
            with self._limits_lock:
                # double check this hasnt been added by another thread already
                if scope not in self._limits:
                    self._limits[scope] = LimitCollection()
                return self._limits[scope]

        return self._limits[scope]

    def wait_until(
        self, region: str, endpoint_name: str, method_name: str
    ) -> datetime.datetime:
        scoped_limit = self.__get_limit(region, endpoint_name, method_name)

        return scoped_limit.wait_until()

    def update_limiter(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        status: int,
        headers: Dict[str, str],
    ):
        raw_limits = self._extract_headers(headers)

        if raw_limits is None:
            return

        scoped_limit = self.__get_limit(region, endpoint_name, method_name)

        scoped_limit.update_limits(raw_limits)

    def _extract_headers(self, headers: Dict[str, str]) -> Optional[List[RawLimit]]:
        limits = HeaderBasedLimiter._extract_single_header(self._limit_header, headers)
        counts = HeaderBasedLimiter._extract_single_header(self._count_header, headers)

        if limits is None or counts is None:
            return None

        if len(limits) != len(counts):
            LOG.warning(
                'header "%s" and "%s" have different sizes!',
                self._limit_header,
                self._count_header,
            )

        combined_limits = list(zip(counts, limits))

        for limit in combined_limits:
            if limit[0][1] != limit[1][1]:
                LOG.warning(
                    " ".join(
                        [
                            'seems that limits for headers "%s" and "%s" did not match',
                            "up correctly! There may be issues in rate limiting.",
                            'Headers were: "%s", "%s".',
                            'Limits from "%s" will be used.',
                        ]
                    ),
                    self._limit_header,
                    self._count_header,
                    headers.get(self._limit_header),
                    headers.get(self._count_header),
                    self._limit_header,
                )

        values = [
            RawLimit(count=limit[0][0], limit=limit[1][0], time=limit[0][1])
            for limit in combined_limits
        ]

        return values

    @staticmethod
    def _extract_single_header(
        header: str, headers: Dict[str, str]
    ) -> Optional[List[List[int]]]:
        values = headers.get(header)

        if values is None:
            return None

        values = values.split(",")

        values = [[int(val) for val in value.split(":")] for value in values]

        return values



from .. import BaseApi, NamedEndpoint
from .urls import ChampionMasteryApiV4Urls


class ChampionMasteryApiV4(NamedEndpoint):
    """
    This class wraps the Champion-Mastery-v4 Api calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#champion-mastery-v4/ for more
    detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new ChampionMasteryApiV4 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_puuid(self, region: str, puuid: str):
        """
        Get all champion mastery entries.

        :param string region:               the region to execute this request on
        :param string puuid:                PUUID associated with the player

        :returns: List[ChampionMasteryDTO]: This object contains a list of Champion
                                            Mastery information for player and champion
                                            combination.
        """
        return self._request_endpoint(
            self.by_puuid.__name__,
            region,
            ChampionMasteryApiV4Urls.by_puuid,
            puuid=puuid,
        )

    def by_puuid_by_champion(self, region: str, puuid: str, champion_id: int):
        """
        Get a champion mastery by player ID and champion ID.

        :param string region:         the region to execute this request on
        :param string puuid:          PUUID associated with the player
        :param long champion_id:      Champion ID to retrieve Champion Mastery for

        :returns: ChampionMasteryDTO: This object contains single Champion Mastery
                                      information for player and champion combination.
        """
        return self._request_endpoint(
            self.by_puuid_by_champion.__name__,
            region,
            ChampionMasteryApiV4Urls.by_puuid_by_champion,
            puuid=puuid,
            champion_id=champion_id,
        )

    def top_by_puuid(self, region: str, puuid: str, count: int = None):
        """
        Get specified number of top champion mastery entries sorted by number of champion
        points descending.

        :param string region:   the region to execute this request on
        :param string puuid:    PUUID associated with the player
        :param int count:       Number of entries to retrieve, defaults to 3.

        :returns: List[ChampionMasteryDto]
        """
        return self._request_endpoint(
            self.top_by_puuid.__name__,
            region,
            ChampionMasteryApiV4Urls.top_by_puuid,
            puuid=puuid,
            count=count,
        )

    def scores_by_puuid(self, region: str, puuid: str):
        """
        Get a player's total champion mastery score, which is the sum of individual
        champion mastery levels

        :param string region:   the region to execute this request on
        :param string puuid:    PUUID of the player

        :returns: int
        """
        return self._request_endpoint(
            self.scores_by_puuid.__name__,
            region,
            ChampionMasteryApiV4Urls.scores_by_puuid,
            puuid=puuid,
        )



from .IllegalArgumentError import IllegalArgumentError
from .RequestHandler import RequestHandler

from .DeprecationHandler import DeprecationHandler
from .DeserializerAdapter import DeserializerAdapter
from .DictionaryDeserializer import DictionaryDeserializer
from .RateLimiterAdapter import RateLimiterAdapter
from .SanitationHandler import SanitationHandler
from .TypeCorrectorHandler import TypeCorrectorHandler
from .ThrowOnErrorHandler import ApiError, ThrowOnErrorHandler



from .LeagueEndpoint import LeagueEndpoint


class SummonerV4Endpoint(LeagueEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/summoner/v4/summoners{url}"
        super().__init__(nurl, **kwargs)


class SummonerApiV4Urls:
    by_account = SummonerV4Endpoint("/by-account/{encrypted_account_id}")
    by_puuid = SummonerV4Endpoint("/by-puuid/{encrypted_puuid}")
    by_id = SummonerV4Endpoint("/{encrypted_summoner_id}")



from .LeagueEndpoint import LeagueEndpoint


class SpecatorV5Endpoint(LeagueEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/spectator/v5{url}"
        super().__init__(nurl, **kwargs)


class SpectatorApiV5Urls:
    by_summoner = SpecatorV5Endpoint(
        "/active-games/by-summoner/{encrypted_puuid}"
    )
    featured_games = SpecatorV5Endpoint("/featured-games")



import logging
from datetime import datetime

from requests import Response

from . import RequestHandler

LOG = logging.getLogger(__name__)


class DeprecationHandler(RequestHandler):
    def __init__(self):
        super().__init__()
        self._warned = set()

    def after_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        response: Response,
    ) -> Response:
        deprecation = response.headers.get("X-Riot-Deprecated")
        if deprecation is not None:
            try:
                deprecation = datetime.utcfromtimestamp(int(deprecation) // 1000)
            except (OSError, ValueError):
                # API returned unexected value in header, so just do nothing
                return response
            key = f"{endpoint_name}.{method_name}"
            if key not in self._warned:
                # technically race condition here, but worst case is we double log
                self._warned.add(key)
                LOG.warning(
                    " ".join(
                        [
                            "Method %s has been deprecated by Riot!",
                            "It will no longer work after %s",
                        ]
                    ),
                    key,
                    deprecation,
                )
        return response



from ... import Endpoint


class DataDragonEndpoint(Endpoint):
    def __init__(self, url, **kwargs):
        nurl = f"https://ddragon.leagueoflegends.com{url}"
        super().__init__(nurl, **kwargs)


class DDragonVersionLocaleEndpoint(DataDragonEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/cdn/{{version}}/data/{{locale}}{url}"
        super().__init__(nurl, **kwargs)


class DataDragonUrls:
    champions = DDragonVersionLocaleEndpoint("/champion.json")
    champions_full = DDragonVersionLocaleEndpoint("/championFull.json")
    items = DDragonVersionLocaleEndpoint("/item.json")
    languages = DDragonVersionLocaleEndpoint("/language.json")
    maps = DDragonVersionLocaleEndpoint("/map.json")
    masteries = DDragonVersionLocaleEndpoint("/mastery.json")
    profile_icons = DDragonVersionLocaleEndpoint("/profileicon.json")
    runes = DDragonVersionLocaleEndpoint("/rune.json")
    runes_reforged = DDragonVersionLocaleEndpoint("/runesReforged.json")
    summoner_spells = DDragonVersionLocaleEndpoint("/summoner.json")
    versions = DataDragonEndpoint("/realms/{region}.json")
    versions_all = DataDragonEndpoint("/api/versions.json")



from .. import BaseApi, NamedEndpoint
from .urls import ClashApiV1Urls


class ClashApiV1(NamedEndpoint):
    """
    This class wraps the Clash-v1 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/apis#clash-v1 for more detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new ClashApiV1 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_summoner(self, region: str, summoner_id: str):
        """
        This endpoint returns a list of active Clash players for a given summoner ID.
        If a summoner registers for multiple tournaments at the same time (e.g.,
        Saturday and Sunday) then both registrations would appear in this list.

        :param string region:               The region to execute this request on.
        :param string summoner_id:          The summoner ID.

        :returns: List[PlayerDTO]: represents the summoner's info for the current clash.
        """
        return self._request_endpoint(
            self.by_summoner.__name__,
            region,
            ClashApiV1Urls.by_summoner,
            summoner_id=summoner_id,
        )

    def by_team(self, region: str, team_id: str):
        """
        Get team by ID.

        :param string region:               The region to execute this request on
        :param string team_id:              Team ID

        :returns: TeamDTO: represents a clash team
        """
        return self._request_endpoint(
            self.by_team.__name__, region, ClashApiV1Urls.by_team, team_id=team_id,
        )

    def tournaments(self, region: str):
        """
        Returns a list of active and upcoming tournaments.

        :param string region:           The region to execute this request on

        :returns: List[TournamentDTO]: represents all of the current tournaments active
        """
        return self._request_endpoint(
            self.tournaments.__name__, region, ClashApiV1Urls.tournaments,
        )

    def tournament_by_team(self, region: str, team_id: str):
        """
        Get tournament by team ID.

        :param string region:                   The region to execute this request on
        :param string team_id:                  Team ID

        :returns: TournamentDTO: represents a clash tournament
        """
        return self._request_endpoint(
            self.tournament_by_team.__name__,
            region,
            ClashApiV1Urls.tournament_by_team,
            team_id=team_id,
        )

    def by_tournament(self, region: str, tournament_id: str):
        """
        Get tournament by ID.

        :param string region:                   The region to execute this request on
        :param string tournament_id:            Tournament ID

        :returns: TournamentDTO: represents a clash tournament
        """
        return self._request_endpoint(
            self.by_tournament.__name__,
            region,
            ClashApiV1Urls.by_tournament,
            tournament_id=tournament_id,
        )



from . import BaseApi

from .Endpoint import Endpoint


class NamedEndpoint:
    """
    Helper class to inject endpoint name into calls to a BaseApi instance without
    the child class explicitly adding the name every time.
    """

    def __init__(self, base_api: BaseApi, endpoint_name: str):
        """
        Initialize a new NamedEndpoint which uses the provided base_api and
        injects the provided endpoint_name into calls to _request

        :param BaseApi base_api:     the root API object to use for making all requests.
        :param string endpoint_name: the name of the child endpoint
        """
        self._base_api = base_api
        self._endpoint_name = endpoint_name

    def _request_endpoint(
        self, method_name: str, region: str, endpoint: Endpoint, **kwargs
    ):
        """
        Sends a request through the BaseApi instance provided, injecting the provided
        endpoint_name into the method call, so the caller doesn't have to.

        :param string method_name:  The name of the calling method
        :param string region:       The region to execute this request on
        :oaram Endpoint endpoint:   The endpoint to use to build URL and query
                                    parameters
        """
        url, query = endpoint(platform=region, **kwargs)
        return self._base_api.raw_request(
            self._endpoint_name, method_name, region, url, query
        )



class IllegalArgumentError(RuntimeError):
    def __init__(
        self, argument_name: str, argument_value: str, extra_message: str = None
    ):
        message = (
            f"Illegal value provided for argument '{argument_name}': '{argument_value}'"
        )
        if extra_message:
            message += f" - {extra_message}"

        super().__init__(message)



from functools import wraps


def remap_region_to_platform(argument_pos: int, argument_name="region"):
    def _remap_region_to_platform(api_call):
        remap = {
            "br1": "americas",
            "la1": "americas",
            "la2": "americas",
            "na1": "americas",
            "oc1": "sea",
            "ph2": "sea",
            "sg2": "sea",
            "th2": "sea",
            "tw2": "sea",
            "vn2": "sea",
            "eun1": "europe",
            "euw1": "europe",
            "ru": "europe",
            "tr1": "europe",
            "jp1": "asia",
            "kr": "asia",
        }

        @wraps(api_call)
        def remapper(*args, **kwargs):
            nargs = args
            if argument_name in kwargs:
                lower_region = kwargs[argument_name].lower()
                if lower_region in remap:
                    kwargs[argument_name] = remap[lower_region]
            if len(args) > argument_pos and isinstance(args[argument_pos], str):
                region = args[1].lower()
                if region in remap:
                    nargs = list(args)
                    nargs[1] = remap[region]
            return api_call(*nargs, **kwargs)

        return remapper

    return _remap_region_to_platform



from .. import BaseApi, NamedEndpoint
from .urls import SummonerApiV4Urls


class SummonerApiV4(NamedEndpoint):
    """
    This class wraps the Summoner-v4 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#summoner-v4 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new SummonerApiV4 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_account(self, region: str, encrypted_account_id: str):
        """
        Get a summoner by account ID.

        :param string region:               The region to execute this request on
        :param string encrypted_account_id: The account ID.

        :returns: SummonerDTO: represents a summoner
        """
        return self._request_endpoint(
            self.by_account.__name__,
            region,
            SummonerApiV4Urls.by_account,
            encrypted_account_id=encrypted_account_id,
        )

    def by_puuid(self, region: str, encrypted_puuid: str):
        """
        Get a summoner by PUUID.

        :param string region:           The region to execute this request on
        :param string encrypted_puuid:  PUUID

        :returns: SummonerDTO: represents a summoner
        """
        return self._request_endpoint(
            self.by_puuid.__name__,
            region,
            SummonerApiV4Urls.by_puuid,
            encrypted_puuid=encrypted_puuid,
        )

    def by_id(self, region: str, encrypted_summoner_id: str):
        """
        Get a summoner by summoner ID.

        :param string region:                   The region to execute this request on
        :param string encrypted_summoner_id:    Summoner ID

        :returns: SummonerDTO: represents a summoner
        """
        return self._request_endpoint(
            self.by_id.__name__,
            region,
            SummonerApiV4Urls.by_id,
            encrypted_summoner_id=encrypted_summoner_id,
        )



from .. import BaseApi, NamedEndpoint
from .urls import LeagueApiV4Urls


class LeagueApiV4(NamedEndpoint):
    """
    This class wraps the League-v4 Api calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#league-v4/ for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new LeagueApiV4 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def challenger_by_queue(self, region: str, queue: str):
        """
        Get the challenger league for a given queue.

        :param string region:   the region to execute this request on
        :param string queue:    the queue to get the challenger players for

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.challenger_by_queue.__name__,
            region,
            LeagueApiV4Urls.challenger_by_queue,
            queue=queue,
        )

    def grandmaster_by_queue(self, region: str, queue: str):
        """
        Get the grandmaster league for a given queue.

        :param string region:   the region to execute this request on
        :param string queue:    the queue to get the grandmaster players for

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.grandmaster_by_queue.__name__,
            region,
            LeagueApiV4Urls.grandmaster_by_queue,
            queue=queue,
        )

    def masters_by_queue(self, region: str, queue: str):
        """
        Get the master league for a given queue.

        :param string region:   the region to execute this request on
        :param string queue:    the queue to get the master players for

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.masters_by_queue.__name__,
            region,
            LeagueApiV4Urls.master_by_queue,
            queue=queue,
        )

    def by_id(self, region: str, league_id: str):
        """
        Get league with given ID, including inactive entries

        :param string region:       the region to execute this request on
        :param string league_id:    the league ID to query

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.by_id.__name__, region, LeagueApiV4Urls.by_id, league_id=league_id
        )

    def by_summoner(self, region: str, encrypted_summoner_id: str):
        """
        Get league entries in all queues for a given summoner ID

        :param string region:                   the region to execute this request on
        :param string encrypted_summoner_id:    the summoner ID to query

        :returns: Set[LeagueEntryDTO]
        """
        return self._request_endpoint(
            self.by_summoner.__name__,
            region,
            LeagueApiV4Urls.by_summoner,
            encrypted_summoner_id=encrypted_summoner_id,
        )

    def by_puuid(self, region: str, puuid: str):
        """
        Get league entries in all queues for a given puuid

        :param string region:                   the region to execute this request on
        :param string puuid:                    the puuid to query

        :returns: Set[LeagueEntryDTO]
        """
        return self._request_endpoint(
            self.by_puuid.__name__,
            region,
            LeagueApiV4Urls.by_puuid,
            puuid=puuid,
        )

    def entries(self, region: str, queue: str, tier: str, division: str, page: int = 1):
        """
        Get all the league entries

        :param string region:   the region to execute this request on
        :param string queue:    the queue to query, i.e. RANKED_SOLO_5x5
        :param string tier:     the tier to query, i.e. DIAMOND
        :param string division: the division to query, i.e. III
        :param int page:        the page for the query to paginate to. Starts at 1.

        :returns: Set[LeagueEntryDTO]
        """
        return self._request_endpoint(
            self.entries.__name__,
            region,
            LeagueApiV4Urls.entries,
            queue=queue,
            tier=tier,
            division=division,
            page=page,
        )



from ... import Endpoint


class DataDragonEndpoint(Endpoint):
    def __init__(self, url, **kwargs):
        nurl = f"https://ddragon.leagueoflegends.com{url}"
        super().__init__(nurl, **kwargs)


class DDragonVersionLocaleEndpoint(DataDragonEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/cdn/{{version}}/data/{{locale}}{url}"
        super().__init__(nurl, **kwargs)


class DataDragonUrls:
    arenas = DDragonVersionLocaleEndpoint("/tft-arena.json")
    augments = DDragonVersionLocaleEndpoint("/tft-augments.json")
    champions = DDragonVersionLocaleEndpoint("/tft-champion.json")
    items = DDragonVersionLocaleEndpoint("/tft-item.json")
    queues = DDragonVersionLocaleEndpoint("/tft-queues.json")
    regalia = DDragonVersionLocaleEndpoint("/tft-regalia.json")
    tacticians = DDragonVersionLocaleEndpoint("/tft-tactician.json")
    traits = DDragonVersionLocaleEndpoint("/tft-trait.json")
    versions = DataDragonEndpoint("/realms/{region}.json")
    versions_all = DataDragonEndpoint("/api/versions.json")



from .InternalLimiter import InternalLimiter
from .HeaderBasedLimiter import HeaderBasedLimiter
from .ApplicationRateLimiter import ApplicationRateLimiter
from .Limits import Limit, LimitCollection, RawLimit
from .MethodRateLimiter import MethodRateLimiter
from .OopsRateLimiter import OopsRateLimiter
from .BasicRateLimiter import BasicRateLimiter

__all__ = [
    "HeaderBasedLimiter",
    "ApplicationRateLimiter",
    "Limit",
    "LimitCollection",
    "RawLimit",
    "MethodRateLimiter",
    "OopsRateLimiter",
    "BasicRateLimiter",
]



from .. import BaseApi, NamedEndpoint
from .urls import SpectatorApiV5Urls


class SpectatorApiV5(NamedEndpoint):
    """
    This class wraps the Spectator-v5 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#spectator-v5 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new SpectatorApiV5 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_summoner(self, region: str, encrypted_puuid: str):
        """
        Get current game information for the given PUUID

        :param string region:                   The region to execute this request on
        :param string encrypted_puuid:          The PUUID of the summoner.

        :returns: CurrentGameInfo
        """
        return self._request_endpoint(
            self.by_summoner.__name__,
            region,
            SpectatorApiV5Urls.by_summoner,
            encrypted_puuid=encrypted_puuid,
        )

    def featured_games(self, region: str):
        """
        Get list of featured games.

        :param string region: The region to execute this request on

        :returns: FeaturedGames
        """
        return self._request_endpoint(
            self.featured_games.__name__, region, SpectatorApiV5Urls.featured_games
        )



import abc
import datetime

from typing import Dict


class InternalLimiter(abc.ABC):
    @abc.abstractmethod
    def wait_until(
        self, region: str, endpoint_name: str, method_name: str
    ) -> datetime.datetime:
        pass

    @abc.abstractmethod
    def update_limiter(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        status: int,
        headers: Dict[str, str],
    ):
        pass



from . import HeaderBasedLimiter


class MethodRateLimiter(HeaderBasedLimiter):
    def __init__(self):
        super().__init__("X-Method-Rate-Limit", "X-Method-Rate-Limit-Count", "Method")

    def _get_limit_scope(
        self, region: str, endpoint_name: str, method_name: str
    ) -> str:
        return f"{region}/{endpoint_name}/{method_name}"



from .. import BaseApi, NamedEndpoint
from .urls import ChallengesApiV1Urls


class ChallengesApiV1(NamedEndpoint):
    """
    This class wraps the Challenges-v1 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#lol-challenges-v1 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new ChallengesApiV1 which uses the provided base_api.

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def config(self, region: str):
        """
        List of all basic challenge configuration information.

        :param string region: The region to execute this request on

        :returns: ChallengeConfigInfoDto
        """
        return self._request_endpoint(
            self.config.__name__, region, ChallengesApiV1Urls.config
        )

    def percentiles(self, region: str):
        """
        Map of level to percentile of players who have achieved it.

        :param string region: The region to execute this request on

        :returns: Map[Long, Map[Integer, Map[Level, Double]]]
        """
        return self._request_endpoint(
            self.percentiles.__name__, region, ChallengesApiV1Urls.percentiles
        )

    def challenge_config(self, region: str, challenge_id: int):
        """
        Get challenge configuration.

        :param string region:                   The region to execute this request on
        
        :param long challenge_id:               The ID of the challenge.

        :returns: ChallengeConfigInfoDto
        """
        return self._request_endpoint(
            self.challenge_config.__name__,
            region,
            ChallengesApiV1Urls.challenge_config,
            challengeId=challenge_id,
        )

    def leaderboards(self, region: str, challenge_id: int, level: str):
        """
        Return top players for each level. Level must be MASTER, GRANDMASTER or CHALLENGER.

        :param string region:                   The region to execute this request on

        :param long challenge_id:               The ID of the challenge.

        :param string level:                    The level to get the leaderboard for.

        :returns: List[ApexPlayerInfoDto]
        """
        return self._request_endpoint(
            self.leaderboards.__name__,
            region,
            ChallengesApiV1Urls.leaderboards,
            challengeId=challenge_id,
            level=level,
        )

    def percentiles_by_challenge_id(self, region: str, challenge_id: int):
        """
        Map of level to percentile of players who have achieved it.

        :param string region:                   The region to execute this request on

        :param long challenge_id:                The ID of the challenge.

        :returns: Map[Level, double]
        """
        return self._request_endpoint(
            self.percentiles_by_challenge_id.__name__,
            region,
            ChallengesApiV1Urls.percentiles_by_challenge_id,
            challengeId=challenge_id,
        )

    def by_puuid(self, region: str, puuid: str):
        """
        Returns player information with list of all progressed challenges.

        :param string region:                   The region to execute this request on

        :param string puuid:                    The puuid.

        :returns: PlayerInfoDto
        """
        return self._request_endpoint(
            self.by_puuid.__name__,
            region,
            ChallengesApiV1Urls.by_puuid,
            puuid=puuid,
        )



import logging
from typing import Union

from .Deserializer import Deserializer
from .RateLimiter import RateLimiter

from .Handlers import (
    DeprecationHandler,
    DeserializerAdapter,
    DictionaryDeserializer,
    RateLimiterAdapter,
    SanitationHandler,
    ThrowOnErrorHandler,
    TypeCorrectorHandler,
)

from .Handlers.RateLimit import BasicRateLimiter

from ._apis import BaseApi, UrlConfig
from ._apis.league_of_legends import (
    ChampionApiV3,
    ChampionMasteryApiV4,
    ClashApiV1,
    DataDragonApi,
    LeagueApiV4,
    LolStatusApiV3,
    LolStatusApiV4,
    SpectatorApiV5,
    SummonerApiV4,
    MatchApiV5,
    ChallengesApiV1,
)

LOG = logging.getLogger(__name__)


class LolWatcher:
    """
    LolWatcher class is intended to be the main interaction point with the APIs for
    League of Legends.
    """

    def __init__(
        self,
        api_key: str = None,
        timeout: int = None,
        kernel_url: str = None,
        rate_limiter: RateLimiter = BasicRateLimiter(),
        deserializer: Deserializer = DictionaryDeserializer(),
        default_status_v4: bool = False,
        **kwargs,
    ):
        """
        Initialize a new instance of the RiotWatcher class.

        :param string api_key: the API key to use for this instance
        :param int timeout: Time to wait for a response before timing out a connection
                            to the Riot API
        :param string kernel_url: URL for the kernel instance to connect to, instead of
                                  the API. See
                                  https://github.com/meraki-analytics/kernel for
                                  details.
        :param RateLimiter rate_limiter: Instance to be used for rate limiting.
                                         This defaults to
                                         Handlers.RateLimit.BasicRateLimiter.
                                         This parameter is not used when connecting to
                                         a kernel instance.
        :param Deserializer deserializer: Instance to be used to deserialize responses
                                          from the Riot Api. Default is
                                          Handlers.DictionaryDeserializer.
        """
        if not kernel_url and not api_key:
            raise ValueError("Either api_key or kernel_url must be set!")

        if kernel_url:
            handler_chain = [
                SanitationHandler(),
                DeserializerAdapter(deserializer),
                ThrowOnErrorHandler(),
                TypeCorrectorHandler(),
                DeprecationHandler(),
            ]
        else:
            handler_chain = [
                SanitationHandler(),
                DeserializerAdapter(deserializer),
                ThrowOnErrorHandler(),
                TypeCorrectorHandler(),
                RateLimiterAdapter(rate_limiter),
                DeprecationHandler(),
            ]

        if kernel_url:
            UrlConfig.root_url = kernel_url
        else:
            UrlConfig.root_url = "https://{platform}.api.riotgames.com"

        self._base_api = BaseApi(api_key, handler_chain, timeout=timeout)

        self._champion = ChampionApiV3(self._base_api)
        self._lol_status_v3 = LolStatusApiV3(self._base_api)
        self._lol_status_v4 = LolStatusApiV4(self._base_api)
        self._data_dragon = DataDragonApi(self._base_api)
        self._clash = ClashApiV1(self._base_api)
        self._champion_mastery = ChampionMasteryApiV4(self._base_api)
        self._league = LeagueApiV4(self._base_api)
        self._match = MatchApiV5(self._base_api)
        self._spectator = SpectatorApiV5(self._base_api)
        self._challenges = ChallengesApiV1(self._base_api)
        self._summoner = SummonerApiV4(self._base_api)

        self._lol_status = (
            self._lol_status_v4 if default_status_v4 else self._lol_status_v3
        )
        # todo: tournament-stub
        # todo: tournament

        if "default_match_v5" in kwargs:
            LOG.warning(
                "property 'default_match_v5' has been deprecated and can be removed"
            )

    @property
    def champion_mastery(self) -> ChampionMasteryApiV4:
        """
        Interface to the ChampionMastery Endpoint

        :rtype: league_of_legends.ChampionMasteryApiV4
        """
        return self._champion_mastery

    @property
    def champion(self) -> ChampionApiV3:
        """
        Interface to the Champion Endpoint

        :rtype: league_of_legends.ChampionApiV3
        """
        return self._champion

    @property
    def clash(self) -> ClashApiV1:
        """
        Interface to the Clash Endpoint

        :rtype: league_of_legends.ClashApiV1
        """
        return self._clash

    @property
    def league(self) -> LeagueApiV4:
        """
        Interface to the League Endpoint

        :rtype: league_of_legends.LeagueApiV4
        """
        return self._league

    @property
    def lol_status(self) -> Union[LolStatusApiV3, LolStatusApiV4]:
        """
        Interface to the LoLStatus Endpoint

        :rtype: league_of_legends.LolStatusApiV3
        """
        return self._lol_status

    @property
    def lol_status_v3(self) -> LolStatusApiV3:
        """
        Interface to the LoLStatus Endpoint

        :rtype: league_of_legends.LolStatusApiV3
        """
        return self._lol_status_v3

    @property
    def lol_status_v4(self) -> LolStatusApiV4:
        """
        Interface to the LoLStatus Endpoint

        :rtype: league_of_legends.LolStatusApiV4
        """
        return self._lol_status_v4

    @property
    def match(self) -> MatchApiV5:
        """
        Interface to the Match Endpoint

        :rtype: league_of_legends.MatchApiV5
        """
        return self._match

    @property
    def match_v4(self):
        """
        This property has been deprecated. Use 'match' property instead.
        Note that v4 is now permanently removed by Riot
        """
        raise NotImplementedError(
            "this property has been deprecated. Use 'match' property instead. Note "
            + "that v4 is now permanently removed by Riot"
        )

    @property
    def match_v5(self):
        """this property has been deprecated. Use 'match' property instead."""
        raise NotImplementedError(
            "this property has been deprecated. Use 'match' property instead."
        )

    @property
    def spectator(self) -> SpectatorApiV5:
        """
        Interface to the Spectator Endpoint

        :rtype: league_of_legends.SpectatorApiV4
        """
        return self._spectator

    @property
    def data_dragon(self) -> DataDragonApi:
        """
        Interface to the DataDragon Endpoint

        :rtype: league_of_legends.DataDragonApi
        """
        return self._data_dragon

    @property
    def summoner(self) -> SummonerApiV4:
        """
        Interface to the Summoner Endpoint

        :rtype: league_of_legends.SummonerApiV4
        """
        return self._summoner

    @property
    def third_party_code(self) -> None:
        """
        DEPRECATED: API has been removed by Riot
        """
        raise NotImplementedError(
            "API has been removed by Riot and no longer functions"
        )

    @property
    def challenges(self) -> ChallengesApiV1:
        """
        Interface to the Challenges Endpoint

        :rtype: league_of_legends.ChallengesApiV1
        """
        return self._challenges



from .. import BaseApi, NamedEndpoint
from .urls import LeagueApiUrls


class LeagueApi(NamedEndpoint):
    """
    This class wraps the Tft-League-v1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#tft-league-v1 for more detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new LeagueApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def challenger(self, region: str):
        """
        Get the challenger league

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.challenger.__name__, region, LeagueApiUrls.challenger
        )

    def by_summoner(self, region: str, encrypted_summoner_id: str):
        """
        Get league entries for a given summoner ID

        :returns: Set[LeagueEntryDTO]
        """
        return self._request_endpoint(
            self.by_summoner.__name__,
            region,
            LeagueApiUrls.by_summoner,
            encrypted_summoner_id=encrypted_summoner_id,
        )

    def entries(self, region: str, tier: str, division: str, page: int = 1):
        """
        Get all the league entries

        :returns: Set[LeagueEntryDTO]
        """
        return self._request_endpoint(
            self.entries.__name__,
            region,
            LeagueApiUrls.entries,
            tier=tier,
            division=division,
            page=page,
        )

    def grandmaster(self, region: str):
        """
        Get the grandmaster league.

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.grandmaster.__name__, region, LeagueApiUrls.grandmaster
        )

    def by_id(self, region: str, league_id: str):
        """
        Get league with given ID, including inactive entries

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.by_id.__name__, region, LeagueApiUrls.by_id, league_id=league_id
        )

    def master(self, region: str):
        """
        Get the master league

        :returns: LeagueListDTO
        """
        return self._request_endpoint(
            self.master.__name__, region, LeagueApiUrls.master
        )

    def rated_ladders(self, region: str, queue: str):
        """
        Get the top rated ladders

        :returns: TopRatedLadderEntryDto
        """
        return self._request_endpoint(
            self.rated_ladders.__name__,
            region,
            LeagueApiUrls.rated_ladders,
            queue=queue,
        )



from .LeagueEndpoint import LeagueEndpoint


class MatchV5Endpoint(LeagueEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/match/v5{url}"
        super().__init__(nurl, **kwargs)


class MatchApiV5Urls:
    by_id = MatchV5Endpoint("/matches/{match_id}")
    matchlist_by_puuid = MatchV5Endpoint(
        "/matches/by-puuid/{puuid}/ids", start=int, count=int, queue=int, type=str, startTime=int, endTime=int
    )
    timeline_by_match = MatchV5Endpoint("/matches/{match_id}/timeline")



from ... import Endpoint, UrlConfig


class TftEndpoint:
    def __init__(self, url: str, **kwargs):
        self._url = f"/tft{url}"

    def __call__(self, **kwargs):
        final_url = f"{UrlConfig.tft_url}{self._url}"

        endpoint = Endpoint(final_url, **kwargs)
        return endpoint(**kwargs)



import pytest

from riotwatcher import LolWatcher, IllegalArgumentError
from riotwatcher._apis.league_of_legends import (
    LolStatusApiV3,
    LolStatusApiV4,
)


@pytest.mark.lol
@pytest.mark.usefixtures("reset_globals")
class TestLolWatcher:
    def test_require_api_key_or_kernel(self):
        with pytest.raises(ValueError):
            LolWatcher()

    def test_allows_positional_api_key(self):
        LolWatcher("RGAPI-this-is-a-fake")

    def test_allows_keyword_api_key(self):
        LolWatcher(api_key="RGAPI-this-is-a-fake")

    def test_allows_kernel_url(self):
        LolWatcher(kernel_url="https://fake-kernel-server")

    def test_defaults_status_v3(self):
        watcher = LolWatcher(api_key="RGAPI-this-is-a-fake")
        assert isinstance(watcher.lol_status, LolStatusApiV3)

    def test_uses_status_v3_when_false(self):
        watcher = LolWatcher(api_key="RGAPI-this-is-a-fake", default_status_v4=False)
        assert isinstance(watcher.lol_status, LolStatusApiV3)

    def test_uses_status_v4_when_true(self):
        watcher = LolWatcher(api_key="RGAPI-this-is-a-fake", default_status_v4=True)
        assert isinstance(watcher.lol_status, LolStatusApiV4)

    def test_stealing_api_key_doesnt_work(self):
        watcher = LolWatcher(api_key="RGAPI-this-is-a-fake")
        with pytest.raises(IllegalArgumentError):
            watcher.lol_status.shard_data("example.com/?stolen-request=")



import pytest

from riotwatcher import TftWatcher, IllegalArgumentError


@pytest.mark.tft
@pytest.mark.usefixtures("reset_globals")
class TestTftWatcher:
    def test_require_api_key(self):
        with pytest.raises(ValueError):
            TftWatcher()

    def test_allows_positional_api_key(self):
        TftWatcher("RGAPI-this-is-a-fake")

    def test_allows_keyword_api_key(self):
        TftWatcher(api_key="RGAPI-this-is-a-fake")

    def test_stealing_api_key_doesnt_work(self):
        watcher = TftWatcher(api_key="RGAPI-this-is-a-fake")
        with pytest.raises(IllegalArgumentError):
            watcher.summoner.by_puuid("example.com/?stolen-request=", "fake-puuid")



import requests

from .Handlers import (
    ApiError as _ApiError,
    IllegalArgumentError as _IllegalArgumentError,
)

ApiError = _ApiError  # should silence code analysis warning
IllegalArgumentError = _IllegalArgumentError
TimeoutError = requests.exceptions.Timeout



import re

from . import IllegalArgumentError, RequestHandler


class SanitationHandler(RequestHandler):
    """
    The SanitationHandler class provides some basic sanity checks to parameters to
    ensure safe usage.

    Only check as of now is ensuring that region doesn't cause HTTP requests to unknown
    servers, which would allow a malicious user to steal API keys.
    """

    def __init__(self):
        super().__init__()
        self._region_expr = re.compile("[a-zA-Z0-9]+")

    def preview_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        query_params: dict,
    ):
        """
        called before a request is processed.

        :param string endpoint_name: the name of the endpoint being requested
        :param string method_name: the name of the method being requested
        :param url: the URL that is being requested.
        :param query_params: dict: the parameters to the url that is being queried,
                                   e.g. ?key1=val&key2=val2
        """
        region_ok = self._region_expr.fullmatch(region)
        if region_ok is None:
            raise IllegalArgumentError("region", region)



class RequestHandler:
    def __init__(self):
        pass

    def preview_request(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        url: str,
        query_params: dict,
    ):
        """
        called before a request is processed.

        :param string region: the region of this request
        :param string endpoint_name: the name of the endpoint being requested
        :param string method_name: the name of the method being requested
        :param url: the URL that is being requested.
        :param query_params: dict: the parameters to the url that is being queried,
                                   e.g. ?key1=val&key2=val2
        """

    def after_request(
        self, region: str, endpoint_name: str, method_name: str, url: str, response
    ):
        """
        Called after a response is received and before it is returned to the user.

        :param string region: the region of this request
        :param string endpoint_name: the name of the endpoint that was requested
        :param string method_name: the name of the method that was requested
        :param url: The url that was requested
        :param response: the response received. This is a response from the "requests"
                         library
        """

    def preview_static_request(self, url: str, query_params: dict):
        """
        Called before a request to DataDragon is processed

        :param url: The url that was requested
        """

    def after_static_request(self, url: str, response):
        """
        Called after a response is received and before it is returned to the user.

        :param url: The url that was requested
        :param response: the response received. This is a response from the "requests"
                         library
        """



import datetime
import logging

from typing import Dict

from .InternalLimiter import InternalLimiter

LOG = logging.getLogger(__name__)


class OopsRateLimiter(InternalLimiter):
    def __init__(self):
        super().__init__()
        self._friendly_name = "429_Limit"
        self._retry_at = None

    @property
    def friendly_name(self) -> str:
        return self._friendly_name

    def wait_until(self, region: str, endpoint_name: str, method_name: str) -> datetime:
        if self._retry_at is not None:
            if self._retry_at > datetime.datetime.now():
                return self._retry_at
            self._retry_at = None
        return datetime.datetime(datetime.MINYEAR, 1, 1)

    def update_limiter(
        self,
        region: str,
        endpoint_name: str,
        method_name: str,
        status: int,
        headers: Dict[str, str],
    ):
        if status == 429:
            retry_after = headers.get("Retry-After")
            limit_type = headers.get("X-Rate-Limit-Type")

            if retry_after is not None:
                LOG.info(
                    'hit 429 from "%s" limit! retrying after %s seconds',
                    limit_type,
                    retry_after,
                )
                self._retry_at = datetime.datetime.now() + datetime.timedelta(
                    seconds=int(retry_after)
                )
            else:
                LOG.info(
                    'hit 429 from "%s" limit! no retry after header...', limit_type
                )



from .. import BaseApi, NamedEndpoint
from .urls import RankedApiUrls


class RankedApi(NamedEndpoint):
    """
    This class wraps the LoR-Ranked-V1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#lor-ranked-v1 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new RankedApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def leaderboards(self, region: str):
        """
        Get the players in Master tier.

        :returns: LeaderboardDto
        """
        return self._request_endpoint(
            self.leaderboards.__name__, region, RankedApiUrls.leaderboards
        )



from .. import BaseApi, NamedEndpoint
from ..helpers import remap_region_to_platform
from .urls import MatchApiUrls

from typing import Optional


class MatchApi(NamedEndpoint):
    """
    This class wraps the TFT-Match-v1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#tft-match-v1 for more detailed information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initializes a new MatchApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    @remap_region_to_platform(1)
    def by_puuid(
        self,
        region: str,
        puuid: str,
        count: int = 20,
        start: int = 0,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
    ):
        """
        Get a list of match ids by PUUID.

        :param string region:   The region to execute this request on
        :param string puuid:    The puuid.
        :param int count:       Defaults to 20. Valid values: 0 to 100. Number of
                                match ids to return.
        :param int start:       Defaults to 0. Start index.
        :param int start_time:  Epoch timestamp in seconds.
        :param int end_time:    Epoch timestamp in seconds.

        :returns: List[string]
        """
        args = {
            "count": count,
            "start": start,
        }

        if start_time:
            args["startTime"] = start_time

        if end_time:
            args["endTime"] = end_time

        return self._request_endpoint(
            self.by_puuid.__name__,
            region,
            MatchApiUrls.by_puuid,
            puuid=puuid,
            **args,
        )

    @remap_region_to_platform(1)
    def by_id(self, region: str, match_id: str):
        """
        Get a match by match id.

        :param string region:   The region to execute this request on
        :param string match_id: The match ID.

        :returns: MatchDto
        """
        return self._request_endpoint(
            self.by_id.__name__, region, MatchApiUrls.by_id, match_id=match_id
        )



import re

from .. import BaseApi, Endpoint
from .urls import DataDragonUrls


class DataDragonApi:
    def __init__(self, base_api: BaseApi):
        self._base_api = base_api

    def arenas(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.arenas, version, locale)

    def augments(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.augments, version, locale)

    def champions(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.champions, version, locale)

    def items(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.items, version, locale)

    def queues(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.queues, version, locale)

    def regalia(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.regalia, version, locale)

    def tacticians(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.tacticians, version, locale)

    def traits(self, version: str, locale: str = None):
        return self._request(DataDragonUrls.traits, version, locale)

    def versions_for_region(self, region: str):
        region = re.sub(r"\d", "", region)
        url, query = DataDragonUrls.versions(region=region)
        return self._base_api.raw_request_static(url, query)

    def versions_all(self):
        url, query = DataDragonUrls.versions_all()
        return self._base_api.raw_request_static(url, query)

    def _request(self, endpoint: Endpoint, version: str, locale: str):
        url, query = endpoint(version=version, locale=locale if locale else "en_US")
        return self._base_api.raw_request_static(url, query)



from .. import BaseApi, NamedEndpoint
from ..helpers import remap_region_to_platform
from .urls import MatchApiV5Urls


class MatchApiV5(NamedEndpoint):
    """
    This class wraps the Match-v5 endpoint calls provided by the Riot API.

    See https://developer.riotgames.com/api-methods/#match-v5 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new MatchApiV5 which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    @remap_region_to_platform(1)
    def by_id(self, region: str, match_id: str):
        """
        Get match by match ID

        :param string region: The region to execute this request on
        :param string match_id: The match ID.

        :returns: MatchDto
        """
        return self._request_endpoint(
            self.by_id.__name__, region, MatchApiV5Urls.by_id, match_id=match_id
        )

    @remap_region_to_platform(1)
    def matchlist_by_puuid(
        self,
        region: str,
        puuid: str,
        start: int = None,
        count: int = None,
        queue: int = None,
        type: str = None,
        start_time: int = None,
        end_time: int = None,
    ):
        """
        Get matchlist for ranked games played on given account ID and platform ID
        and filtered using given filter parameters, if any

        :param string region:   The region to execute this request on
        :param string puuid:    The puuid.
        :param int start:       Defaults to 0. Start index.
        :param int count:       Defaults to 20. Valid values: 0 to 100. Number of
                                match ids to return.
        :param int queue:       Filter the list of match ids by a specific queue id.
                                This filter is mutually inclusive of the type filter
                                meaning any match ids returned must match both the
                                queue and type filters.
        :param string type:     Filter the list of match ids by the type of match.
                                This filter is mutually inclusive of the queue filter
                                meaning any match ids returned must match both the
                                queue and type filters.
        :param long start_time: Epoch timestamp in seconds. The matchlist started
                                storing timestamps on June 16th, 2021. Any matches
                                played before June 16th, 2021 won't be included in the
                                results if the startTime filter is set.
        :param long end_time:   Epoch timestamp in seconds.

        :returns: List[string]
        """
        return self._request_endpoint(
            self.matchlist_by_puuid.__name__,
            region,
            MatchApiV5Urls.matchlist_by_puuid,
            puuid=puuid,
            start=start,
            count=count,
            queue=queue,
            type=type,
            startTime=start_time,
            endTime=end_time,
        )

    @remap_region_to_platform(1)
    def timeline_by_match(self, region: str, match_id: str):
        """
        Get match timeline by match ID.

        Not all matches have timeline data.

        :param string region: The region to execute this request on
        :param string match_id: The match ID.

        :returns: MatchTimelineDto
        """
        return self._request_endpoint(
            self.timeline_by_match.__name__,
            region,
            MatchApiV5Urls.timeline_by_match,
            match_id=match_id,
        )



from .LeagueEndpoint import LeagueEndpoint


class ChallengesApiV1Endpoints(LeagueEndpoint):
    def __init__(self, url, **kwargs):
        nurl = f"/challenges/v1{url}"
        super().__init__(nurl, **kwargs)


class ChallengesApiV1Urls:
    config = ChallengesApiV1Endpoints("/challenges/config")
    percentiles = ChallengesApiV1Endpoints("/challenges/percentiles")
    challenge_config = ChallengesApiV1Endpoints("/challenges/{challengeId}/config")
    leaderboards = ChallengesApiV1Endpoints("/challenges/{challengeId}/leaderboards/by-level/{level}")
    percentiles_by_challenge_id = ChallengesApiV1Endpoints("/challenges/{challengeId}/percentiles")
    by_puuid = ChallengesApiV1Endpoints("/player-data/{puuid}")



from .BaseApi import BaseApi
from .NamedEndpoint import NamedEndpoint

from .Endpoint import Endpoint
from .UrlConfig import UrlConfig

from . import (
    league_of_legends,
    legends_of_runeterra,
    riot,
    team_fight_tactics,
    valorant,
)

__all__ = [
    "BaseApi",
    "NamedEndpoint",
    "Endpoint",
    "UrlConfig",
    "league_of_legends",
    "legends_of_runeterra",
    "riot",
    "team_fight_tactics",
    "valorant",
]



from .. import BaseApi, NamedEndpoint
from .urls import ContentApiUrls


class ContentApi(NamedEndpoint):
    """
    This class wraps the Val-Content-v1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#val-content-v1 for more detailed
    information
    """

    def __init__(self, base_api: BaseApi):
        """
        Initialize a new ContentApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def contents(self, region: str, locale: str = None):
        """
        Get content optionally filtered by locale

        :returns: ContentDto
        """
        return self._request_endpoint(
            self.contents.__name__, region, ContentApiUrls.contents, locale=locale
        )



from .. import BaseApi, NamedEndpoint
from .urls import SummonerApiUrls


class SummonerApi(NamedEndpoint):
    """
    This class wraps the TFT-Summoner-v1 Api calls provided by the Riot API.

    See https://developer.riotgames.com/apis#tft-summoner-v1 for more detailed
    information.
    """

    def __init__(self, base_api: BaseApi):
        """
        Initializes a new SummonerApi which uses the provided base_api

        :param BaseApi base_api: the root API object to use for making all requests.
        """
        super().__init__(base_api, self.__class__.__name__)

    def by_account(self, region: str, encrypted_account_id: str):
        """
        Get a summoner by account ID

        :returns: SummonerDTO
        """
        return self._request_endpoint(
            self.by_account.__name__,
            region,
            SummonerApiUrls.by_account,
            encrypted_account_id=encrypted_account_id,
        )

    def by_name(self, region: str, summoner_name: str):
        """
        Get a summoner by summoner name.

        :returns: SummonerDTO
        """
        return self._request_endpoint(
            self.by_name.__name__,
            region,
            SummonerApiUrls.by_name,
            summoner_name=summoner_name,
        )

    def by_puuid(self, region: str, puuid: str):
        """
        Get a summoner by PUUID.

        :returns: SummonerDTO
        """
        return self._request_endpoint(
            self.by_puuid.__name__, region, SummonerApiUrls.by_puuid, puuid=puuid
        )

    def by_id(self, region: str, encrypted_summoner_id: str):
        """
        Get a summoner by summoner ID

        :returns: SummonerDTO
        """
        return self._request_endpoint(
            self.by_id.__name__,
            region,
            SummonerApiUrls.by_id,
            encrypted_summoner_id=encrypted_summoner_id,
        )
